@@ -272,7 +272,7 @@ def test_assert_match_existing_snapshot_is_not_file(testdir, basic_case_dir):
     'all-bytes',
     'unicode',
     'whitespace',
-    'slash-r'
+    'slash-r',
 ])
 def test_assert_match_edge_cases(testdir, basic_case_dir, tested_value):
     """
@@ -332,7 +332,7 @@ def test_assert_match_unsupported_value_update_existing_snapshot(testdir, basic_
         from unittest import mock
         def _file_encode(string: str) -> bytes:
             return string.replace('\n', os.linesep).encode()
-        
+
         def test_sth(snapshot):
             snapshot.snapshot_dir = 'case_dir'
             with mock.patch('pytest_snapshot.plugin._file_encode', _file_encode):
@@ -352,7 +352,7 @@ def test_assert_match_unsupported_value_create_snapshot(testdir, basic_case_dir)
         from unittest import mock
         def _file_encode(string: str) -> bytes:
             return string.replace('\n', os.linesep).encode()
-        
+
         def test_sth(snapshot):
             snapshot.snapshot_dir = 'case_dir'
             with mock.patch('pytest_snapshot.plugin._file_encode', _file_encode):
@@ -375,6 +375,6 @@ def test_assert_match_unsupported_value_slash_r(testdir, basic_case_dir):
     result = testdir.runpytest('-v', '--snapshot-update')
     result.stdout.fnmatch_lines([
         '*::test_sth FAILED*',
-        'E* ValueError: Snapshot testing strings containing "\\r" is not supported.'
+        'E* ValueError: Snapshot testing strings containing "\\r" is not supported.',
     ])
     assert result.ret == 1
